# (1): Select proper energy eigenvalues file by commenting/uncommenting one of the lines below

eneFileName = str("%s.energyso" %(WorkingDir.split('/')[-1]))
#eneFileName = str("%s.energyhfso" %(WorkingDir.split('/')[-1]))

# (2): Edit the band range at the end of this file
#       for m in range(48,50): # conduction
#           ...
#           for n in range(42,48): # valence
#
# (3) Execution:
#       cd wien2k_case_directory
#       python /path/to/dplmelmt.py

import os
import sys
import numpy as np

WorkingDir = os.getcwd()
print (f'Working directory = {WorkingDir}')
mmnFileName = str("%s.mmn" %(WorkingDir.split('/')[-1]))
print (f'mmnFileName = {mmnFileName}')
print (f'eneFileName = {eneFileName}')
klistFileName = str("%s.klist_band" %(WorkingDir.split('/')[-1]))
print (f'klistFileName = {klistFileName}')
outpdFileName = str("%s.outputd" %(WorkingDir.split('/')[-1]))
print (f'outpdFileName = {outpdFileName}')
structFileName = str("%s.struct" %(WorkingDir.split('/')[-1]))
print (f'structFileName = {structFileName}')

# read case.struct file
# determine number of non-equivalent atoms 
lineindex = 0
with open(structFileName) as infile:
    for line in infile:
        if(lineindex == 1):
            print(f'line({lineindex})={line}')
            cut = line.split() # split line with spaces
            print(f'cut({lineindex})={cut}')
            nat = int(cut[2])
            break # finish reading
        lineindex += 1 # count lines read

# read case.mmn file
lineindex = 0
nmax = float("inf") # set it first, then change later
with open(mmnFileName) as infile:
    for line in infile:
        if(lineindex == 0):
            print(f'line({lineindex})={line}')
        elif(lineindex == 1):
            cut = line.split() # split line with spaces
            if(not len(cut) == 3):
                print(f'line {lineindex} split = {cut}, while expected 3 items')
                raise # error
            nmax = int(cut[0])
            # Allocate Mmn matrix elements
            mmn = np.zeros((nmax**2,1),dtype=np.complex_)
        elif(lineindex == 2):
            pass # skip the line
        elif(lineindex == nmax**2 + 3):
            break # finish reading
        else: # all lines, but 0-2
            cut = line.split() # split line with spaces
            if(not len(cut) == 2):
                print(f'line {lineindex} split = {cut}, while expected 2 items')
                raise # error
            i = lineindex - 3
            mmn[i] = float(cut[0]) + float(cut[1])*1j # make a complex number
        lineindex += 1 # count lines read

# read energies
lineindex = 0
nmaxene1 = float("inf") # set it first, then change later
nmaxene2 = float("inf")
with open(eneFileName) as infile:
    for line in infile:
        if(lineindex in range(2*nat)):
            pass # skip lines
        elif(lineindex == 2*nat):
            cut = line.split() # split line with spaces
            nmaxene1 = int(cut[-2])
            ene1 = np.zeros(nmaxene1) # vector of energies (Ry)
        elif(lineindex == nmaxene1 + 2*nat + 1):
            cut = line.split() # split line with spaces
            nmaxene2 = int(cut[-2])
            ene2 = np.zeros(nmaxene2) # vector of energies (Ry)
        elif(lineindex == nmaxene1 + nmaxene2 + 2*nat + 2):
            break # finish reading
        else: # all lines, but 0-4
            cut = line.split() # split line with spaces
            if(not len(cut) == 2):
                print(f'line {lineindex} split = {cut}, while expected 2 items')
                raise # error
            if(nmaxene2 == float("inf")):
                i = lineindex - (2*nat + 1)
                ene1[i] = float(cut[1]) # (Ry)
            else:
                i = lineindex - (2*nat + 2) - nmaxene1
                ene2[i] = float(cut[1]) # (Ry)
        lineindex += 1 # count lines read
ene1 = ene1/2. # Ry -> Ha
ene2 = ene2/2.

# check nmax
if(nmax > nmaxene1 or nmax > nmaxene2):
    print(f'nmax={nmax} should be less than number of eigenvalues {nmaxene1} and {nmaxene2} in {eneFileName} file')
    raise # error

# reshape Mmn matrix
mmn = np.reshape(mmn, (nmax, nmax), order='C')

# determine wave vector q in units of BR1_REC
lineindex = 0
with open(klistFileName) as infile:
    for line in infile:
        if('END' in line): # end of file
            break
        elif(lineindex == 0): # k-point 1
            line1 = line
        elif(lineindex == 1): # k-point 2
            line2 = line
        else:
            print(f'file {klistFileName}, line {lineindex} = {line}, \
                    while END is expected')
            raise # error
        lineindex += 1 # count lines read
if( lineindex != 2 ):
    raise # the case.klist_band file should have exactly 2 k points
# read k-points coordinates as formated input
line = line1 # line in case.klist_band
if( len(line2) < 45 ): # short format klist; we use 2nd line to check length!
    divisor = float(line[25:30])
    qx = float(line[10:15])/divisor
    qy = float(line[15:20])/divisor
    qz = float(line[20:25])/divisor
else: # long format klist
    divisor = float(line[40:50])
    qx = float(line[10:20])/divisor
    qy = float(line[20:30])/divisor
    qz = float(line[30:40])/divisor
q1 = np.array([qx,qy,qz])
line = line2
if( len(line2) < 45 ): # short format klist
    divisor = float(line[25:30])
    qx = float(line[10:15])/divisor
    qy = float(line[15:20])/divisor
    qz = float(line[20:25])/divisor
else: # long format klist
    divisor = float(line[40:50])
    qx = float(line[10:20])/divisor
    qy = float(line[20:30])/divisor
    qz = float(line[30:40])/divisor
q2 = np.array([qx,qy,qz])

# read rec lattice vectors BR1_REC (2pi/bohr) in case.outputd
read = False
linesread = 0
br1rec = np.zeros((3,3)) # allocate recip. lattice vectors
with open(outpdFileName) as infile:
    for line in infile:
        if('-BR1_REC-' in line): # find pattern for beggining of the matrix
            read = True # next 3 lines should be read
        elif(read): # start reading after -BR1_REC- is found
            cut = line.split() # split line with spaces
            if(not len(cut) == 3):
                print(f'line {lineindex} split = {cut}, while expected 3 items')
                raise # error
            br1rec[linesread,0] = float(cut[0])
            br1rec[linesread,1] = float(cut[1])
            br1rec[linesread,2] = float(cut[2])
            linesread += 1
        if(linesread > 2):
            break # exit the loop after 3 lines are read
print(f'br1rec (2pi/bohr)={br1rec}')

# determine wave vector q (2pi/bohr)
q = q2 - q1
print(f'q={q}')
br1rectr = np.transpose(br1rec)
q = np.matmul(br1rectr,q)
qabs = np.linalg.norm(q)
print(f'|q| = {qabs} (2pi/bohr)')

# velocity matrix elements
Vmn = np.zeros((nmax,nmax),dtype=np.complex_) # allocate recip. lattice vectors
for m in range(nmax):
    for n in range(nmax):
        Vmn[m,n] =  mmn[m,n]*(ene2[n]-ene1[m])/qabs

for m in range(48,50): # conduction
    print(f'ene1[{m}] = {ene1[m]} Ha')
    for n in range(42,48): # valence
        #print(f'ene2[{n}] = {ene2[n]} Ha')
        #print(f'ene[{n}] = {ene[n]} Ha')
        #print(f'mmn[{m},{n}] = [{mmn[m,n].real} {mmn[m,n].imag}j]')
        #print(f'|mmn[{m},{n}]/qabs|**2 = [{(abs(mmn[m,n])/qabs)**2}]')
        #print(f'Vmn[{m},{n}] = [{Vmn[m,n].real} {Vmn[m,n].imag}j]')
        print(f'|Vmn[{m},{n}]|**2 = [{abs(Vmn[m,n])**2}]')
