#!/bin/bash

case=${PWD##*/}
numk=300
red=3
ecut=-8
nbmax=50
lvns=6
rkmax=8
kshift=0

source ~/.bashrc-w2k_21.1b-mpi
export EDITOR=cat # prevent openning files for editing

# clean up
shopt -s extglob # get spectial rm command
rm -r !(slurm-${SLURM_JOBID}*) # remove all files/dir except "slurm-XXXX...*"
shopt -u extglob # disable extension

# initialization
cp ../${case}.struct ${case}.struct
init_lapw -b -vxc 13 -red $red -ecut $ecut -rkmax $rkmax -lvns $lvns -numk $numk
echo -e "${numk}\n${kshift}" | x kgen -so # non-shifted k mesh

# prepare machines file
echo SLURM_CPUS_PER_TASK=$SLURM_CPUS_PER_TASK
echo "1:localhost" > .machines # 1st line
for (( i=2; i<=$SLURM_CPUS_PER_TASK; i++ ))
do
  echo "1:localhost" >> .machines # remaining N-1 lines
done
echo "granularity:1" >> .machines
echo "extrafine:1" >> .machines
echo SLURM_TASKS_PER_NODE=$SLURM_TASKS_PER_NODE
export OMP_NUM_THREADS=$SLURM_TASKS_PER_NODE
echo OMP_NUM_THREADS=$OMP_NUM_THREADS

# run without SOC
run_lapw -ec 0.0001 -cc 0.001 -p
save_lapw -d noSOC

# init SOC
echo -e "0 0 1\n\n10.0\n\nN\n" | init_so_lapw # Emax=10 Ry in *.in1(c)
sed -i 's/^-10  1\.9/-10  7.0/g' ${case}.inso # Emax=7 Ry for SO

# run with SOC
run_lapw -ec 0.0001 -cc 0.001 -so -s lapw1 -p

# check gap
grep -e :GAP *scf
save_lapw -d PBESOC

# init HF
init_hf_lapw -s
echo -e "${numk}\n${kshift}" | run_kgenhf_lapw -so # same k mesh for HF
sed -i "s/^xx /${nbmax}/g" ${case}.inhf # put upper number of bands

# run hybrid in parallel
run_lapw -ec 0.0001 -cc 0.001 -so -hf -p

# check gap
grep -e :GAP *scf
